package example;

import java.io.FileOutputStream;
import java.text.SimpleDateFormat;

import jp.co.systembase.core.DataTable;
import jp.co.systembase.report.Report;
import jp.co.systembase.report.ReportPages;
import jp.co.systembase.report.ReportSetting;
import jp.co.systembase.report.component.ElementDesign;
import jp.co.systembase.report.data.ReportDataSource;
import jp.co.systembase.report.renderer.pdf.PdfRenderer;
import jp.co.systembase.report.renderer.pdf.PdfRendererSetting;
import jp.co.systembase.report.textformatter.ITextFormatter;
import jp.co.systembase.report.textformatter.TextFormatterUtil;

public class ExampleExtention {

	public static void main(String[] args) throws Exception {
		ReportSetting setting = new ReportSetting();
		setting.textFormatterMap.replace("default", new TextFormatterEx());

		Report report = new Report(ReadUtil.readJson("report/example_extention.rrpt"), setting);
		report.fill(new ReportDataSource(getDataTable()));
		ReportPages pages = report.getPages();

		// PDF出力
		{
			PdfRendererSetting pdfSetting = new PdfRendererSetting();
			FileOutputStream fos = new FileOutputStream("output/example_extention.pdf");
			try {
				PdfRenderer renderer = new PdfRenderer(fos, pdfSetting);
				pages.render(renderer);
			} finally {
				fos.close();
			}
		}
	}

	private static DataTable getDataTable() throws Exception {
		DataTable ret = new DataTable();
		SimpleDateFormat sdf = new SimpleDateFormat("yyyy/MM/dd");
		ret.setFieldNames("v");

		ret.addRecord().puts(sdf.parse("2019/01/01"));
		ret.addRecord().puts(sdf.parse("2019/04/30"));
		ret.addRecord().puts(sdf.parse("2019/05/01"));
		ret.addRecord().puts(sdf.parse("2020/01/01"));
		ret.addRecord().puts(sdf.parse("2021/01/01"));
		ret.addRecord().puts(sdf.parse("2022/01/01"));
		ret.addRecord().puts(sdf.parse("2032/01/01"));

		return ret;
	}

	public static class TextFormatterEx implements ITextFormatter
	{
		public String format(Object v, ElementDesign design)
		{
			String _v = TextFormatterUtil.format(v, design);
			Boolean b = (Boolean)design.get("fullchar");
		    if (b instanceof Boolean){
				if ((Boolean)b){
					// 全角に変換
					return toFullWidth(_v);
				}
		    }
			return _v.toString();
		}

		private static String toFullWidth(String s){
			StringBuilder sb = new StringBuilder(s);
			for (int i = 0; i < s.length(); i++){
				char c = s.charAt(i);
				if (0x30 <= c && c <= 0x39) {
					sb.setCharAt(i, (char) (c + 0xFEE0));
				}
			}
			return sb.toString();
		}
	}
}
